window.Gleam.initSearch([{"doc":"gliew","title":"gliew","content":"# gliew\n\n[![Package Version](https://img.shields.io/hexpm/v/gliew)](https://hex.pm/packages/gliew)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/gliew/)\n\nAn **experimental** web framework with server-side rendering and optional live updating UI (à la Phoenix&#39;s LiveView) for gleam.\n\n## Quick start\n\nUsage is similar to [mist](https://hexdocs.pm/mist/) except the handler function should return a [nakai](https://hexdocs.pm/nakai/) node tree.\n\n```gleam\nimport gleam/erlang/process\nimport gleam/http.{Get}\nimport gleam/http/request\nimport nakai/html\nimport gliew\n\npub fn main() {\n  let assert Ok(_) =\n    gliew.new(\n      8080,\n      fn(req) {\n        case req.method, request.path_segments(req) {\n          Get, [&quot;hello&quot;] -&gt;\n            html.div_text([], &quot;Hello gleam!&quot;)\n            |&gt; gliew.view(200)\n          _, _ -&gt;\n            html.div_text([], &quot;Hello world!&quot;)\n            |&gt; gliew.view(200)\n        }\n      },\n    )\n    |&gt; gliew.serve\n\n  process.sleep_forever()\n}\n```\n\n### Live mounts\n\nIn order to have some elements of the HTML live updating you will need to use live mounts.\n\nThey can be either used by calling the `gliew.live_mount` function directly:\n\n```gleam\nimport gleam/int\nimport gleam/option.{Option}\nimport gleam/erlang/process.{Subject}\nimport gleam/http/request\nimport nakai/html\nimport gliew\n\npub fn main() {\n  let assert Ok(_) =\n    gliew.new(\n      8080,\n      fn(req) {\n        case req.method, request.path_segments(req) {\n          _, _ -&gt;\n            html.div(\n              [],\n              [\n                html.div_text([], &quot;counter is at:&quot;),\n                gliew.live_mount(\n                  mount: mount_counter,\n                  with: Nil,\n                  render: render_counter,\n                ),\n              ],\n            )\n            |&gt; gliew.view(200)\n        }\n      },\n    )\n    |&gt; gliew.serve\n\n  process.sleep_forever()\n}\n\n// The render function is called with `None` on\n// initial render but `Some(a)` every time there\n// is a new value on the subject returned by the\n// mount function.\nfn render_counter(assign: Option(Int)) {\n  html.div_text(\n    [],\n    assign\n    |&gt; option.unwrap(0)\n    |&gt; int.to_string,\n  )\n}\n\n// This is the mount function.\nfn mount_counter(_ctx) {\n  let subject = process.new_subject()\n\n  let _ = process.start(fn() { loop(subject, 0) }, True)\n\n  subject\n}\n\n// Counter loop for demonstration purposes.\nfn loop(subject: Subject(Int), current: Int) {\n  process.send(subject, current)\n  process.sleep(1000)\n  loop(subject, current + 1)\n}\n```\n\nBut a nicer way (IMO) is to use the `use` syntax to turn a function into a live mount by default.\n\n```gleam\nimport gleam/int\nimport gleam/option\nimport gleam/erlang/process.{Subject}\nimport gleam/http/request\nimport nakai/html\nimport gliew\n\npub fn main() {\n  let assert Ok(_) =\n    gliew.new(\n      8080,\n      fn(req) {\n        case req.method, request.path_segments(req) {\n          _, _ -&gt;\n            html.div([], [html.div_text([], &quot;counter is at:&quot;), counter()])\n            |&gt; gliew.view(200)\n        }\n      },\n    )\n    |&gt; gliew.serve\n\n  process.sleep_forever()\n}\n\n// This becomes the render function.\nfn counter() {\n  // Here we use the `use` syntax as kind of a decorator\n  // to turn this function into a live mount.\n  use assign &lt;- gliew.live_mount(mount_counter, with: Nil)\n\n  html.div_text(\n    [],\n    assign\n    |&gt; option.unwrap(0)\n    |&gt; int.to_string,\n  )\n}\n\n// This is still the mount function.\nfn mount_counter(_ctx) {\n  let subject = process.new_subject()\n\n  let _ = process.start(fn() { loop(subject, 0) }, True)\n\n  subject\n}\n\n// Counter loop for demonstartion purposes.\nfn loop(subject: Subject(Int), current: Int) {\n  process.send(subject, current)\n  process.sleep(1000)\n  loop(subject, current + 1)\n}\n```\n\n## Example\n\nThe following example has a random string generator and a global counter so that every browser session remains in sync when navigating to `http://localhost:8080/mounts`.\n\n```gleam\nimport gleam/int\nimport gleam/string\nimport gleam/base\nimport gleam/option.{None, Some}\nimport gleam/http.{Get, Post}\nimport gleam/http/request\nimport gleam/erlang/process.{Subject}\nimport gleam/crypto.{strong_random_bytes}\nimport nakai/html\nimport nakai/html/attrs\nimport gliew\n// This is only here for this example.\n// It implements an actor that keeps an\n// incrementing counter and can be\n// subscribed to.\nimport gliew/integration/counter.{\n  CounterMessage, reset, start_counter, subscribe,\n}\n\npub fn main() {\n  // Start anything that will be needed by the handlers.\n  // In this case we start an actor that simply increments\n  // a counter (starting at 0).\n  // You can subscribe to the counter to be notified every\n  // time it increments.\n  let assert Ok(count_actor) = start_counter()\n\n  // Start the gliew server.\n  // This is a thin wrapper around mist which handles\n  // rendering your views and starting workers for\n  // live connections.\n  let assert Ok(_) =\n    gliew.new(\n      8080,\n      fn(req) {\n        case req.method, request.path_segments(req) {\n          Get, [&quot;mounts&quot;] -&gt;\n            mounts_page(count_actor)\n            |&gt; gliew.view(200)\n          Post, [&quot;counter&quot;, &quot;reset&quot;] -&gt; reset_counter(count_actor)\n          _, _ -&gt;\n            home_page()\n            |&gt; gliew.view(200)\n        }\n      },\n    )\n    |&gt; gliew.serve\n\n  process.sleep_forever()\n}\n\n// You can just return a simple nakai node tree for\n// a static page.\nfn home_page() {\n  html.div_text([], &quot;Hello gleam!&quot;)\n}\n\nfn reset_counter(count_actor: Subject(CounterMessage)) {\n  reset(count_actor)\n\n  gliew.response(204)\n}\n\nfn mounts_page(count_actor: Subject(CounterMessage)) {\n  // Return the HTML for the page.\n  html.div(\n    [\n      attrs.style(\n        [\n          &quot;display: flex&quot;, &quot;flex-direction: column&quot;, &quot;justify-content: center&quot;,\n          &quot;align-items: center&quot;, &quot;row-gap: 1em&quot;, &quot;height: 100vh&quot;,\n        ]\n        |&gt; string.join(&quot;;&quot;),\n      ),\n    ],\n    [\n      // Random text\n      html.div_text([attrs.style(&quot;font-size: x-large&quot;)], &quot;Random text for you&quot;),\n      random_text(),\n      // Counter\n      html.div_text([attrs.style(&quot;font-size: x-large&quot;)], &quot;Counter is at&quot;),\n      counter(count_actor),\n      html.div(\n        [],\n        [\n          html.button_text([], &quot;Reset&quot;)\n          |&gt; gliew.on_click(do: Post, to: &quot;/counter/reset&quot;),\n        ],\n      ),\n      // Explanation text\n      html.div(\n        [\n          attrs.style(\n            [\n              &quot;background-color: #d0ebf4&quot;, &quot;padding: 1em&quot;, &quot;color: #222&quot;,\n              &quot;border-radius: 1em&quot;, &quot;width: 21em&quot;, &quot;margin-top: 0.5em&quot;,\n            ]\n            |&gt; string.join(&quot;;&quot;),\n          ),\n        ],\n        [\n          html.div_text(\n            [attrs.style(&quot;font-size: x-large; margin-bottom: 0.5em;&quot;)],\n            &quot;This page is live rendered.&quot;,\n          ),\n          html.div_text(\n            [attrs.style(&quot;font-size: large&quot;)],\n            &quot;Once the browser has made a connection to the server a new random text is generated every 5 seconds and the counter above should auto-increment.&quot;,\n          ),\n        ],\n      ),\n    ],\n  )\n}\n\n//                           //\n// Global counter live mount //\n//                           //\n\n// A live mount that shows a live updating counter.\nfn counter(count_actor: Subject(CounterMessage)) {\n  // Makes the function a live mount.\n  // The second parameter passed to `gliew.live_mount` will\n  // be passed to the mount function (first parameter)\n  // when the mount is mounted (i.e. the client connects\n  // back to the server to get live updates).\n  //\n  // The returned value is of type `Option(a)` where\n  // `a` is the type in the Subject returned by mount.\n  use assign &lt;- gliew.live_mount(counter_mount, with: count_actor)\n\n  let text = case assign {\n    // View has been mounted and we want to use the\n    // &quot;live&quot; value that was setup in the mount function.\n    Some(counter) -&gt;\n      counter\n      |&gt; int.to_string\n    // View is being rendered on the server before\n    // the client has made a connection back to the server.\n    // Here we contact the count_actor to get the current\n    // value.\n    None -&gt; &quot;Loading..&quot;\n  }\n\n  // Return the HTML of the view.\n  html.div_text([attrs.style(&quot;font-size: xx-large&quot;)], text)\n}\n\n// A mount function that runs once the client connects through\n// a websocket.\n// This should be used to subscribe to whatever data that should\n// be returned to the render function and return the subject.\nfn counter_mount(count_actor: Subject(CounterMessage)) {\n  // Create a new subject.\n  let subject = process.new_subject()\n\n  // Subscribe to counter.\n  subscribe(count_actor, subject)\n\n  // Return the subject.\n  subject\n}\n\n//                        //\n// Random text live mount //\n//                        //\n\n// A live mount that renders random text on an interval.\nfn random_text() {\n  use assign &lt;- gliew.live_mount(text_mount, with: Nil)\n\n  html.div_text(\n    [attrs.style(&quot;font-size: xx-large&quot;)],\n    assign\n    |&gt; option.unwrap(random_string()),\n  )\n}\n\n// The mount function used for the random_text mount.\n// It will generate random text on an interval and\n// send it on the returned subject.\nfn text_mount(_) {\n  let subject = process.new_subject()\n\n  let _ = process.start(fn() { loop(subject) }, True)\n\n  subject\n}\n\n// Random text loop.\nfn loop(subject: Subject(String)) {\n  process.send(subject, random_string())\n  process.sleep(5000)\n  loop(subject)\n}\n\n// Helper function to generate random string.\nfn random_string() {\n  strong_random_bytes(10)\n  |&gt; base.encode64(False)\n}\n````\n\nThe above example implementation can be found in `test/integration/live_mounts.gleam` and quickly run with:\n\n```sh\ngleam run -m gliew/integration/live_mounts\n```\n\nAnd then navigate to `http://localhost:8080/mounts` to see in action.\n\n## Installation\n\nIf available on Hex this package can be added to your Gleam project:\n\n```sh\ngleam add gliew\n```\n\nand its documentation can be found at &lt;https://hexdocs.pm/gliew&gt;.\n","url":"index.html"},{"doc":"gliew","title":"Response","content":"pub opaque type Response\n\n\nSynonyms:\ngliew.Response\ngliew Response","url":"gliew.html#Response"},{"doc":"gliew","title":"Server","content":"pub type Server {\n  Server(\n    port: Int,\n    layout: fn(html.Node(Event)) -&gt; html.Node(Event),\n    handler: fn(Request(Body)) -&gt; Response,\n  )\n}\n Configuration for a server.\n\nServer(\n  port: Int,\n  layout: fn(html.Node(Event)) -&gt; html.Node(Event),\n  handler: fn(Request(Body)) -&gt; Response,\n)\n\n\nSynonyms:\ngliew.Server\ngliew Server","url":"gliew.html#Server"},{"doc":"gliew","title":"append","content":"pub fn append() -&gt; Attr(Event)\n Append the innerHTML of the live mount instead of\n replacing it.\n\nSynonyms:\ngliew.append\ngliew append","url":"gliew.html#append"},{"doc":"gliew","title":"live_mount","content":"pub fn live_mount(mount mount: fn(a) -&gt; Subject(b), with context: a, render render: fn(\n    Option(b),\n  ) -&gt; Node(Event)) -&gt; Node(Event)\n Creates a HTML node tree that will be `mounted`\n and receive live updates with data on `Subject(a)`\n returned by the mount function.\n\nSynonyms:\ngliew.live_mount\ngliew live_mount","url":"gliew.html#live_mount"},{"doc":"gliew","title":"morph","content":"pub fn morph() -&gt; Attr(Event)\n Makes the live mount morph the diff of the innerHTML\n instead of replacing the whole thing.\n\nSynonyms:\ngliew.morph\ngliew morph","url":"gliew.html#morph"},{"doc":"gliew","title":"new","content":"pub fn new(port: Int, handler: fn(Request(Body)) -&gt; Response) -&gt; Server\n Creates a configuration for a server with default\n layout.\n\nSynonyms:\ngliew.new\ngliew new","url":"gliew.html#new"},{"doc":"gliew","title":"on_click","content":"pub fn on_click(node: Node(Event), do method: Method, to path: String) -&gt; Node(\n  Event,\n)\n Attaches an on-click handler to a node by specifying\n the HTTP method and path to make a request to when\n clicked.\n\nSynonyms:\ngliew.on_click\ngliew on_click","url":"gliew.html#on_click"},{"doc":"gliew","title":"response","content":"pub fn response(status: Int) -&gt; Response\n Creates an empty response without a body.\n\nSynonyms:\ngliew.response\ngliew response","url":"gliew.html#response"},{"doc":"gliew","title":"script","content":"pub fn script() -&gt; Node(a)\n Returns script tags to put into your own layout in order\n for live mounts to work.\n\nSynonyms:\ngliew.script\ngliew script","url":"gliew.html#script"},{"doc":"gliew","title":"serve","content":"pub fn serve(server: Server) -&gt; Result(Nil, StartError)\n Start server.\n\nSynonyms:\ngliew.serve\ngliew serve","url":"gliew.html#serve"},{"doc":"gliew","title":"view","content":"pub fn view(node: Node(Event), status: Int) -&gt; Response\n Creates a view response.\n\nSynonyms:\ngliew.view\ngliew view","url":"gliew.html#view"},{"doc":"gliew","title":"with_body","content":"pub fn with_body(response: Response, body: String) -&gt; Response\n Sets body of a response.\n\nSynonyms:\ngliew.with_body\ngliew with_body","url":"gliew.html#with_body"},{"doc":"gliew","title":"with_header","content":"pub fn with_header(response: Response, key key: String, value value: String) -&gt; Response\n Adds a header to a response.\n\nSynonyms:\ngliew.with_header\ngliew with_header","url":"gliew.html#with_header"},{"doc":"gliew","title":"gliew","content":"","url":"gliew.html"}]);